package io.github.palexdev.validationdemo;

import io.github.palexdev.validationdemo.validation.Severity;
import io.github.palexdev.validationdemo.validation.automatic.AutomaticConstraint;
import io.github.palexdev.validationdemo.validation.automatic.AutomaticConstraintBuilder;
import io.github.palexdev.validationdemo.validation.automatic.AutomaticValidator;
import javafx.application.Application;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.CheckBox;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.Region;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;

import java.util.List;

public class ValidationTest extends Application {

	@Override
	public void start(Stage primaryStage) {
		BorderPane borderPane = new BorderPane();

		VBox errorsContainer = new VBox(10);
		errorsContainer.setAlignment(Pos.TOP_CENTER);
		errorsContainer.setMaxSize(Region.USE_PREF_SIZE, Region.USE_PREF_SIZE);

		TextField field = new TextField();
		field.setPromptText("8 characters minimum");
		field.setMaxSize(Region.USE_PREF_SIZE, Region.USE_PREF_SIZE);

		CheckBox checkBox = new CheckBox("Agree");

		Button button = new Button("Validate");

		VBox container = new VBox(20, errorsContainer, field, checkBox, button);
		container.setAlignment(Pos.CENTER);

		AutomaticConstraint textLength = AutomaticConstraintBuilder.build()
				.setSeverity(Severity.ERROR)
				.setMessage("Text should be at least 8 characters long!")
				.setCondition(field.textProperty().length().greaterThanOrEqualTo(8))
				.get();
		AutomaticConstraint agree = AutomaticConstraintBuilder.build()
				.setSeverity(Severity.ERROR)
				.setMessage("Must agree!")
				.setCondition(checkBox.selectedProperty())
				.get();

		AutomaticValidator validator = new AutomaticValidator()
				.addConstraint("text length", textLength)
				.addConstraint("agree", agree);

		validator.setOnInvalid(() -> showErrors(errorsContainer, validator));

		// This enables automatic validation
		// Note that for automatic validation is also needed to use a custom
		// property (ValidProperty) to force fireValueChangedEvent() in the
		// validator's update() method
		/*
		textLength.setOnInvalidated(value -> validator.update());
		agree.setOnInvalidated(value -> validator.update());
		*/

		button.setOnAction(event -> showErrors(errorsContainer, validator));

		borderPane.setCenter(container);
		Scene scene = new Scene(borderPane, 800, 800);
		primaryStage.setScene(scene);
		primaryStage.show();
	}

	private void showErrors(VBox errorsContainer, AutomaticValidator validator) {
		errorsContainer.getChildren().clear();
		List<AutomaticConstraint> invalidConstraints = validator.validate();
		invalidConstraints.forEach(constraint -> errorsContainer.getChildren().add(new Label(constraint.getMessage())));
	}
}
