package io.github.palexdev.validationdemo.validation;

import java.util.function.Predicate;

public class ConstraintBuilder<T> {
	private final Constraint<T> constraint = new Constraint<>();

	private ConstraintBuilder() {}

	public static <T> ConstraintBuilder<T> build() {
		return new ConstraintBuilder<>();
	}

	public ConstraintBuilder<T> setSeverity(Severity severity) {
		constraint.setSeverity(severity);
		return this;
	}

	public ConstraintBuilder<T> setMessage(String message) {
		constraint.setMessage(message);
		return this;
	}

	public ConstraintBuilder<T> setCondition(Predicate<T> condition) {
		constraint.setCondition(condition);
		return this;
	}

	public Constraint<T> get() {
		checkConstraint();
		return constraint;
	}

	private void checkConstraint() {
		Severity severity = constraint.getSeverity();
		String message = constraint.getMessage();
		Predicate<T> condition = constraint.getCondition();

		if (severity == null) throw new IllegalArgumentException("Severity not set!");
		if (message == null) throw new IllegalArgumentException("Message not set!");
		if (condition == null) throw new IllegalArgumentException("Condition not set!");
	}
}
