package io.github.palexdev.validationdemo.validation.automatic;

import io.github.palexdev.validationdemo.others.OnInvalidated;
import io.github.palexdev.validationdemo.others.When;
import io.github.palexdev.validationdemo.validation.Severity;
import javafx.beans.binding.BooleanExpression;

import java.util.function.Consumer;

public class AutomaticConstraint {
	private Severity severity;
	private String message;
	private BooleanExpression condition;
	private Consumer<Boolean> action;
	private boolean started = false;

	AutomaticConstraint() {}

	public AutomaticConstraint(Severity severity, String message, BooleanExpression condition) {
		this.severity = severity;
		this.message = message;
		this.condition = condition;
		start();
	}

	public static AutomaticConstraint of(Severity severity, String message, BooleanExpression condition) {
		return new AutomaticConstraint(severity, message, condition);
	}

	protected void onInvalidated(boolean value) {
		if (action != null) action.accept(value);
	}

	public void start() {
		if (!started) {
			OnInvalidated.on(condition).then(this::onInvalidated).listen();
			started = true;
		}
	}

	public void stop() {
		When.disposeFor(condition);
		started = false;
	}

	public boolean isValid() {
		return condition.get();
	}

	public Severity getSeverity() {
		return severity;
	}

	void setSeverity(Severity severity) {
		this.severity = severity;
	}

	public String getMessage() {
		return message;
	}

	void setMessage(String message) {
		this.message = message;
	}

	public BooleanExpression conditionProperty() {
		return condition;
	}

	void setCondition(BooleanExpression condition) {
		this.condition = condition;
	}

	public void setOnInvalidated(Consumer<Boolean> action) {
		this.action = action;
	}
}
