package io.github.palexdev.validationdemo.validation.automatic;

import io.github.palexdev.validationdemo.validation.Severity;
import javafx.beans.binding.BooleanExpression;

import java.util.function.Consumer;

public class AutomaticConstraintBuilder {
	private final AutomaticConstraint constraint = new AutomaticConstraint();

	public static AutomaticConstraintBuilder build() {
		return new AutomaticConstraintBuilder();
	}

	public AutomaticConstraintBuilder setSeverity(Severity severity) {
		constraint.setSeverity(severity);
		return this;
	}

	public AutomaticConstraintBuilder setMessage(String message) {
		constraint.setMessage(message);
		return this;
	}

	public AutomaticConstraintBuilder setCondition(BooleanExpression condition) {
		constraint.setCondition(condition);
		return this;
	}

	public AutomaticConstraintBuilder setOnInvalidated(Consumer<Boolean> action) {
		constraint.setOnInvalidated(action);
		return this;
	}

	public AutomaticConstraint get() {
		checkConstraint();
		constraint.start();
		return constraint;
	}

	private void checkConstraint() {
		Severity severity = constraint.getSeverity();
		String message = constraint.getMessage();
		BooleanExpression condition = constraint.conditionProperty();

		if (severity == null) throw new IllegalArgumentException("Severity not set!");
		if (message == null) throw new IllegalArgumentException("Message not set!");
		if (condition == null) throw new IllegalArgumentException("Condition not set!");
	}
}
