package io.github.palexdev.validationdemo.validation2;

import io.github.palexdev.validationdemo.validation.Severity;
import javafx.beans.binding.BooleanExpression;

public class Constraint {
	private Severity severity;
	private String message;
	private BooleanExpression condition;

	protected Constraint() {}

	public Constraint(Severity severity, String message, BooleanExpression condition) {
		this.severity = severity;
		this.message = message;
		this.condition = condition;
	}

	public static Constraint of(Severity severity, String message, BooleanExpression condition) {
		return new Constraint(severity, message, condition);
	}

	public boolean isValid() {
		return condition.getValue();
	}

	public Severity getSeverity() {
		return severity;
	}

	protected void setSeverity(Severity severity) {
		this.severity = severity;
	}

	public String getMessage() {
		return message;
	}

	protected void setMessage(String message) {
		this.message = message;
	}

	public BooleanExpression getCondition() {
		return condition;
	}

	protected void setCondition(BooleanExpression condition) {
		this.condition = condition;
	}

	public static class Builder {
		private final Constraint constraint = new Constraint();

		public static Builder build() {
			return new Builder();
		}

		public Builder setSeverity(Severity severity) {
			constraint.setSeverity(severity);
			return this;
		}

		public Builder setMessage(String message) {
			constraint.setMessage(message);
			return this;
		}

		public Builder setCondition(BooleanExpression condition) {
			constraint.setCondition(condition);
			return this;
		}

		public Constraint get() {
			checkConstraint();
			return constraint;
		}

		private void checkConstraint() {
			Severity severity = constraint.getSeverity();
			String message = constraint.getMessage();
			BooleanExpression condition = constraint.getCondition();

			if (severity == null) throw new IllegalArgumentException("Severity not set!");
			if (message == null) throw new IllegalArgumentException("Message not set!");
			if (condition == null) throw new IllegalArgumentException("Condition not set!");
		}
	}
}
